CREATE SEQUENCE seq_segler;
CREATE SEQUENCE seq_tour INCREMENT BY 10 MINVALUE 1000 NO CYCLE;
CREATE TYPE segelschein AS ENUM ('A','B','C');

CREATE TABLE Segler (
    id INTEGER PRIMARY KEY DEFAULT nextval('seq_segler'),
	name VARCHAR(40) NOT NULL,
	gebdatum DATE NOT NULL,
	notfallkontakt VARCHAR(200)
);

CREATE TABLE Koch (
	id INTEGER PRIMARY KEY REFERENCES Segler(id),
	seehauben INTEGER NOT NULL
);

CREATE TABLE Ersthelfer (
	id INTEGER PRIMARY KEY REFERENCES Segler(id),
	ausbildung VARCHAR(200) NOT NULL
);

CREATE TABLE Skipper (
	id INTEGER PRIMARY KEY REFERENCES Segler(id),
	schein segelschein NOT NULL,
	seemeilen INTEGER NOT NULL,
	lieblingstour INTEGER
);

CREATE TABLE Crew (
    leiter INTEGER REFERENCES Skipper(id),
	cid INTEGER,
	crewname VARCHAR(10) NOT NULL,
	PRIMARY KEY(leiter,cid)
);

CREATE TABLE CrewMitglieder (
    segler INTEGER REFERENCES Segler(id),
	cleiter INTEGER NOT NULL,
	cid INTEGER NOT NULL,
	FOREIGN KEY (cleiter, cid) REFERENCES Crew(leiter, cid),
	PRIMARY KEY (segler, cleiter, cid)
);

--crewleiter duerfen nur crewmitglieder werden
ALTER TABLE Crew ADD CONSTRAINT fk_crewskipper FOREIGN KEY (leiter,leiter,cid) REFERENCES CrewMitglieder(segler,cleiter,cid) DEFERRABLE INITIALLY DEFERRED;

CREATE TABLE CrewErsthelfer (
    ersthelfer INTEGER REFERENCES Ersthelfer(id),
	cleiter INTEGER NOT NULL,
	cid INTEGER NOT NULL,
	FOREIGN KEY (cleiter,cid) REFERENCES Crew(leiter, cid),
	FOREIGN KEY (ersthelfer,cleiter,cid) REFERENCES CrewMitglieder(segler,cleiter,cid),
	PRIMARY KEY (ersthelfer, cleiter, cid)
);

CREATE TABLE CrewKoch (
    koch INTEGER REFERENCES Koch(id), 
	cleiter INTEGER NOT NULL,
	cid INTEGER NOT NULL,
	FOREIGN KEY (cleiter, cid) REFERENCES Crew(leiter, cid),
    FOREIGN KEY (koch,cleiter,cid) REFERENCES CrewMitglieder(segler,cleiter,cid),
	PRIMARY KEY (koch, cleiter, cid)
);

CREATE TABLE Ankerplatz (
    krz CHAR(3) PRIMARY KEY,
	koordinaten VARCHAR(10) NOT NULL,
	name VARCHAR(20) NOT NULL
);

CREATE TABLE Tankstelle (
    krz CHAR(3) PRIMARY KEY REFERENCES Ankerplatz(krz),
	selfservice BOOLEAN NOT NULL
);

CREATE TABLE Fjord (
    krz CHAR(3) PRIMARY KEY REFERENCES Ankerplatz(krz),
	nacht BOOLEAN NOT NULL,
	beschreibung TEXT NOT NULL
);

CREATE TABLE Hafen (
    krz CHAR(3) PRIMARY KEY REFERENCES Ankerplatz(krz),
	gebuehr NUMERIC(6,2) NOT NULL,
	duschen BOOLEAN NOT NULL,
	einkauf BOOLEAN NOT NULL
);

CREATE TABLE Segelschiff (
    hafen VARCHAR(3) REFERENCES Hafen(krz),
	sid INTEGER NOT NULL,
	name VARCHAR(20) NOT NULL,
	baujahr VARCHAR(4) NOT NULL,
	kojen INTEGER NOT NULL CHECK (kojen >= 0),
	kapazitaet INTEGER NOT NULL CHECK (kapazitaet >= 0),
	PRIMARY KEY (hafen, sid)
);

CREATE TABLE Tour (
    tid INTEGER PRIMARY KEY DEFAULT nextval('seq_tour'),
	bez VARCHAR(20) NOT NULL,
	dauer INTEGER NOT NULL,
	geplantvon INTEGER NOT NULL REFERENCES Skipper(id)
);

CREATE TABLE Teiltour (
    tid INTEGER REFERENCES Tour(tid),
	teilvon INTEGER REFERENCES Tour(tid),
	PRIMARY KEY (tid, teilvon)
);


CREATE TABLE TourAnkerplatz (
    tid INTEGER REFERENCES Tour(tid),
	krz CHAR(3) REFERENCES Ankerplatz(krz),
	tag INTEGER CHECK (tag > 0),
	PRIMARY KEY (tid,krz,tag),
    UNIQUE (tid,tag)
);

CREATE TABLE Gechartert (
    cleiter INTEGER,
	cid INTEGER,
	hafen CHAR(3),
	sid INTEGER,
	tid INTEGER REFERENCES Tour(tid),
	startdatum DATE NOT NULL,
	FOREIGN KEY (cleiter, cid) REFERENCES Crew(leiter, cid),
	FOREIGN KEY (hafen, sid) REFERENCES Segelschiff(hafen, sid),
	PRIMARY KEY (cleiter, cid, hafen, sid, tid, startdatum)
);

ALTER TABLE Skipper ADD CONSTRAINT fk_lieblingstour FOREIGN KEY (lieblingstour) REFERENCES Tour(tid) DEFERRABLE INITIALLY DEFERRED;
